%Copyright (C) 2005  Carvalho-Mariel

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function bw = bwhitmiss(varargin)

%BWHITMISS Binary hit-miss operation.
%   BW2 = BWHITMISS(BW,SE1,SE2) performs the hit-miss operation defined
%   by the structuring elements SE1 and SE2.  The hit-miss operation
%   preserves pixels whose neighborhoods "match" the shape of SE1 and
%   don't match the shape of SE2.    The domains of SE1 and SE2 should not have
%   any common elements.  BWHITMISS(BW,SE1,SE2) is equivalent to
%   IMERODE(BW,SE1) & IMERODE(~BW,SE2).
%
%   BW2 = BWHITMISS(BW,INTERVAL) performs the hit-miss operation defined
%   in terms of a single array, called an "interval."  An interval is an
%   array whose elements can be either 1, 0, or -1.  The 1-valued
%   elements make up the domain of SE1; the -1-valued elements make up
%   the domain of SE2; and the 0-valued elements are ignored.
%   BWHITMISS(INTERVAL) is equivalent to BWHITMISS(BW,INTERVAL == 1,
%   INTERVAL == -1). 
%
%   Class support
%   -------------
%   BW1 can be a nonsparse numeric array of any class and dimension.  BW2
%   is always a logical uint8 array with the same size as BW1.  SE1 and
%   SE2 must be flat STREL objects or arrays containing 1s and 0s.
%   INTERVAL must be an array containing 1s, 0s, and -1s. 
%
%   Example
%   -------
%       bw = [0 0 0 0 0 0
%             0 0 1 1 0 0
%             0 1 1 1 1 0
%             0 1 1 1 1 0
%             0 0 1 1 0 0
%             0 0 1 0 0 0]
%
%       interval = [0 -1 -1
%                   1  1 -1
%                   0  1  0];
%
%       bw2 = bwhitmiss(bw,interval)

	if(nargin != 2 && nargin != 3)
		usage("bw2 = bwhitmiss(img_bin, SE1, SE2) OU bw2 = bwhitmiss(img_bin, interval)");
	elseif(nargin == 2)
		% BWHITMISS(BW,INTERVAL)
    		interval = varargin{2};
    		if ~isnumeric(interval) | ~isreal(interval)
        		error('INTERVAL doit etre une matrice de reels.');
    		endif
    		if ~all((interval(:) == 1) | (interval(:) == 0) | (interval(:) == -1))
        		error('INTERVAL doit contenir seulement -1s, 0s, et 1s.');
    		end
		
		SE1 = (interval == 1);
		SE2 = (interval == -1);
	else
		SE1=varargin{2};
		SE2=varargin{3};
		if ~all((SE1(:) == 1) | (SE1(:) == 0))
        		error('SE1 doit contenir seulement 0s, et 1s.');	
		elseif ~all((SE2(:) == 1) | (SE2(:) == 0))
        		error('SE2 doit contenir seulement 0s, et 1s.');	
		endif
	endif
	
	img_bin=varargin{1};

	bw = erode(img_bin, SE1) & erode(~img_bin, SE2);

endfunction
