% Copyright (C) 2005  Barre-Piquot
%
%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function [seuil] = graythresh (I)
%level = graythresh (I)  
% Compute global image threshold using Otsu's method 
%
% level = graythresh(I) computes a global threshold (level) that can be used to convert an intensity image to a binary image with im2bw. 
% 
% level is a normalized intensity value that lies in the range [0, 1]. 
% 
% The graythresh function uses Otsu's method, which chooses the threshold to minimize the intraclass variance of the black and white pixels. 
% 
% Multidimensional arrays are converted automatically to 2-D arrays. The return value level is a double scalar.
% 
%       img=imread(filename);
%       seuil=graythresh(img)
%       BW = im2bw(img,seuil);
%       imshow(BW)
%
% See Also im2bw 

        s=size(I);
	img=zeros(s(1),s(2));
	if (length(s)==3)
		%Conversion of color image to a gray image
		img =( (I(:,:,1)*0.299).+(I(:,:,2)*0.587).+(I(:,:,3)*0.114));
	else
		img=I;
	end
	
	%Calculation of the normalized histogram
	h=hist(img(:),256);        
	h=h/(length(img(:))+1);
	
	w=zeros(256,1);
	mu=zeros(256,1);
	h=h(:);

	%Calculation of the cumulated histogram and the mean values
	w(1)=h(1);
	mu(1)=h(1);
	for i=2:256
		w(i)=h(i)+w(i-1);
		mu(i)=mu(i-1) + i*h(i);
	end	
     	
	%initialisation of the values used for the threshold calculation
	w0=w(1);
	w1=1-w0;
	mu0=mu(1)/w0;
	mu1=(mu(256)-mu(1))/w1;
	max=w0*w1*(mu1-mu0)*(mu1-mu0);
	seuil=1	;
	
	%For each step of the histogram, calculation of the threshold and storing of the maximum
	for i=2:256
		w0=w(i);
		w1=1-w0;
		mu0=mu(i)/w0;
		mu1=(mu(256)-mu(i))/w1;
		s=w0*w1*(mu1-mu0)*(mu1-mu0);
		if(s>max)
			max=s;
			seuil=i;
		endif
	end
	
	%Normalisation of the threshold		
     	seuil=seuil/256;
	
endfunction

